@@ -9,6 +9,10 @@ from eccez import QueryError
 The Mother DB engine.
 """
 
+DB_POOL_LIMITED = 1
+DB_POOL_ELASTIC = 2
+DB_POOL_GROWING = 3
+
 def _connectStr(loc):
 
     dbhost= loc['DB_HOST']
@@ -234,6 +238,9 @@ class PsygresFly(Speaker):
 
         return
 
+    def _close(self):
+        self.pg_conn.close()
+
     def _import(self, cls):
         for m in PsygresFly.exported_methods:
             setattr(cls, m, getattr(self, m))
@@ -339,10 +346,10 @@ class PsygresFly(Speaker):
                     "Broken session removd from the pool.",
                     ERR_COL(self.session_name), ERR_COL(res))
 
+        #self.log_debug(
+        #        "Session %s returning to the Pool...", 
+        #        OKI_COL(self.session_name))
         PsygresPool.backHome(self)
-        self.log_debug(
-                "Session %s back to the Pool.", 
-                OKI_COL(self.session_name))
 
     def rollback(self):
         self.log_debug("Rollbacking queries for Session %s", 
@@ -361,7 +368,6 @@ class PsygresFly(Speaker):
 
 class PsygresPool:
 
-    import threading
     from Queue import Empty as _empty_queue
 
     _pool_initialized= False
@@ -372,6 +378,10 @@ class PsygresPool:
     _pool_current= 0
     _pool_orphans= {}
     _pool_queue= None
+    _pool_type= DB_POOL_LIMITED
+    _pool_calm= False
+
+    import threading
 
     _pool_current_mutex= threading.Lock()
     _pool_get_mutex= threading.Lock()
@@ -379,6 +389,16 @@ class PsygresPool:
 
     del threading
 
+    @staticmethod
+    def _pool_type_str(ptype):
+
+        if ptype == DB_POOL_LIMITED:
+            return 'Limited'
+        elif ptype == DB_POOL_ELASTIC: 
+            return 'Elastic'
+        else:
+            return 'Growing'
+
     @staticmethod
     def _add_orphan(sname, db):
         m= PsygresPool._pool_orphans_mutex
@@ -395,6 +415,13 @@ class PsygresPool:
 
     @staticmethod
     def _full():
+
+        ptype= PsygresPool._pool_type
+
+        if ptype in [DB_POOL_ELASTIC, DB_POOL_GROWING]:
+            return False
+
+        # Db Pool is Limited
         m= PsygresPool._pool_current_mutex
         m.acquire()
         res= PsygresPool._pool_current== PsygresPool._pool_max
@@ -403,8 +430,9 @@ class PsygresPool:
 
     @staticmethod
     def status():
-        """ status() -> (available, total, min, max, orphaned)
+        """ status() -> (pool_type, available, total, min, max, orphaned)
         """
+
         mc= PsygresPool._pool_current_mutex
         mg= PsygresPool._pool_get_mutex
         mo= PsygresPool._pool_orphans_mutex
@@ -417,45 +445,56 @@ class PsygresPool:
         mo.release()
         mg.release()
         mc.release()
-        max= PsygresPool._pool_max
         min= PsygresPool._pool_min
-        return (available, total, min, max, orphaned)
+        max= PsygresPool._pool_max
+
+        ptype= PsygresPool._pool_type
+        sptype= PsygresPool._pool_type_str(ptype)
+
+        return (sptype, available, total, min, max, orphaned)
 
     @staticmethod
-    def _add_conn(n=1, pg_conn=None):
+    def _add_conn(n=1):
         """ Never call me directly: use _addConnection!"""
-        max= PsygresPool._pool_max
+        #max= PsygresPool._pool_max
+        #cur= PsygresPool._pool_current
+        #dif= max- cur
+
+        #if not dif:
+        #    Speaker.log_warning(
+        #            "No space left in PsygresPool "
+        #            "(max= %s)", ERR_COL(max))
+        #    return dif
+
+        #elif dif< n:
+        #    Speaker.log_warning(
+        #            "Adding only %s new connection(s) "
+        #            "(max= %s, cur= %s, req= %s)",
+        #            OKI_COL(dif), ERR_COL(max), 
+        #            INF_COL(cur), INF_COL(n))
+
+        #else:
+        #    dif= n
+        #    Speaker.log_info(
+        #            "Adding %s new connection(s) (max= %s, cur= %s)",
+        #            OKI_COL(dif), OKI_COL(max), OKI_COL(cur))
+
         cur= PsygresPool._pool_current
-        dif= max- cur
+        max= PsygresPool._pool_max 
 
-        if not dif:
-            Speaker.log_warning(
-                    "No space left in PsygresPool "
-                    "(max= %s)", ERR_COL(max))
-            return dif
+        Speaker.log_info(
+                "Adding %s new connection(s) (max= %s, cur= %s)",
+                OKI_COL(n), OKI_COL(max), OKI_COL(cur))
 
-        elif dif< n:
-            Speaker.log_warning(
-                    "Adding only %s new connection(s) "
-                    "(max= %s, cur= %s, req= %s)",
-                    OKI_COL(dif), ERR_COL(max), 
-                    INF_COL(cur), INF_COL(n))
-
-        else:
-            dif= n
-            Speaker.log_info(
-                    "Adding %s new connection(s) (max= %s, cur= %s)",
-                    OKI_COL(dif), OKI_COL(max), OKI_COL(cur))
-
-        for i in xrange(dif):
-            p= PsygresFly(pg_conn)
+        for i in xrange(n):
+            p= PsygresFly()
             q= PsygresPool._pool_queue
             p._pool_queue= q
             q.put(p)
 
-        PsygresPool._pool_current+= dif
+        PsygresPool._pool_current+= n
 
-        return dif
+        return n
 
     ##
     # Note that this function could be called by:
@@ -464,13 +503,13 @@ class PsygresPool:
     #                  lock is acquired.
     ##
     @staticmethod
-    def _addConnection(n=1, pg_conn= None):
+    def _addConnection(n=1):
         """ 
         Don't call me directly, use newSession() instead!
         """
         m= PsygresPool._pool_current_mutex
         m.acquire()
-        PsygresPool._add_conn(n, pg_conn)
+        PsygresPool._add_conn(n)
         m.release()
 
 
@@ -482,7 +521,7 @@ class PsygresPool:
         Speaker.log_info("Initializing connection Pool ...")
         from Queue import Queue
         PsygresPool._pool_queue= Queue()
-        PsygresPool._addConnection(n, pg_conn)
+        PsygresPool._addConnection(n)
 
         PsygresPool._pool_initialized= True
 
@@ -498,14 +537,46 @@ class PsygresPool:
         except PsygresPool._empty_queue:
             pass
 
-        if PsygresPool._full():
-            return db_pool.get(True, PsygresPool._pool_timeout)
+        full= PsygresPool._full()
+        calm= PsygresPool._pool_calm
 
+        # Wait or create immediately a new connection?
+        if calm or full:
+            
+            # wait.
+            ptimeout= PsygresPool._pool_timeout
+            Speaker.log_info(
+                "DbPool: waiting for a free connection "
+                "(timeout= %s) ...", INF_COL(ptimeout))
+
+            # If full, wait and hope.
+            if full:
+                return db_pool.get(True, ptimeout)
+
+            # If calm, wait only
+            try:
+                return db_pool.get(True, ptimeout)
+            except PsygresPool._empty_queue:
+                pass
+        
+        # Ok, here the pool is not full.
+        # Moreover, we have already waited.
         PsygresPool._addConnection()
-
         return db_pool.get_nowait()
 
 
+        #if PsygresPool._full():
+        #    ptimeout= PsygresPool._pool_timeout
+        #    Speaker.log_info(
+        #            "DbPool is full: waiting for a free connection "
+        #            "(timeout= %s) ...", INF_COL(ptimeout))
+        #    return db_pool.get(True, ptimeout)
+
+        #PsygresPool._addConnection()
+
+        #return db_pool.get_nowait()
+
+
     @staticmethod
     def newSession(name= None, pg_conn= None):
 
@@ -518,7 +589,7 @@ class PsygresPool:
         m.acquire()
 
         try:
-            db= PsygresPool._get_session()
+            session= PsygresPool._get_session()
 
         except Exception, ss:
             m.release()
@@ -528,12 +599,12 @@ class PsygresPool:
 
         sname= name or PsygresFly._default_session
 
-        PsygresPool._add_orphan(sname, db)
+        PsygresPool._add_orphan(sname, session)
 
         m.release()
-        db.session_name= sname 
+        session.session_name= sname 
 
-        return db
+        return session
 
     @staticmethod
     def backHome(pg_conn):
@@ -544,18 +615,40 @@ class PsygresPool:
 
         m.acquire()
 
+        if PsygresPool._pool_type == DB_POOL_ELASTIC and \
+           PsygresPool._pool_current > PsygresPool._pool_min:
+                PsygresPool._ns_discard(pg_conn)
+                #PsygresPool._del_orphan(pg_conn)
+                #pg_conn._close()
+                #PsygresPool._pool_current-= 1
+                Speaker.log_info("Elastic Pool: session %s "
+                                 "closed and removed.", OKI_COL(sname))
+                m.release()
+                return
+
         try:
             q.put_nowait(pg_conn)
         except Exception, ss:
-            PsygresPool._pool_current-= 1
             Speaker.log_warning(
-                    "Removed connection %s from Pool: %s.", 
+                    "Removing connection %s from Pool: %s.", 
                     ERR_COL(pg_conn.session_name), ERR_COL(ss))
+            PsygresPool._ns_discard(pg_conn)
+            m.release()
+            return
+            #pg_conn._close()
+            #PsygresPool._pool_current-= 1
 
         PsygresPool._del_orphan(pg_conn)
-
+        Speaker.log_info('Session %s back to the Pool.', OKI_COL(sname))
         m.release()
+        return
 
+    @staticmethod
+    def _ns_discard(pg_conn):
+
+        PsygresPool._pool_current-= 1
+        PsygresPool._del_orphan(pg_conn)
+        pg_conn._close()
 
     @staticmethod
     def discard(pg_conn):
@@ -563,8 +656,10 @@ class PsygresPool:
         m= PsygresPool._pool_current_mutex
 
         m.acquire()
-        PsygresPool._pool_current-= 1
-        PsygresPool._del_orphan(pg_conn)
+        PsygresPool._ns_discard(pg_conn)
+        #PsygresPool._pool_current-= 1
+        #PsygresPool._del_orphan(pg_conn)
+        #pg_conn._close()
         m.release()
         Speaker.log_warning("Connection %s dropped from Pool.", 
                 ERR_COL(pg_conn.session_name))
@@ -572,7 +667,7 @@ class PsygresPool:
 
 def init_dbda(conf, pg_conn= None):
 
-    if not hasattr(Psygres, '_db_initialized'):
+    if Psygres._db_initialized:
         Speaker.log_info("Dbda already initialized.")
         return
 
@@ -580,8 +675,10 @@ def init_dbda(conf, pg_conn= None):
     if not isinstance(conf, dict):
         import speaker
         loc= {}
+        names_dict= speaker.__dict__.copy()
+        names_dict.update(globals())
         try:
-            execfile(conf, speaker.__dict__, loc)
+            execfile(conf, names_dict, loc)
         except Exception, ss:
             err= "Unable to read Mother configuration file %s: %s" % (ERR_COL(conf), ss)
     else:
@@ -594,27 +691,40 @@ def init_dbda(conf, pg_conn= None):
     if err:
         Speaker.log_int_raise(err)
 
-    Psygres._connect_str= _connectStr(loc)
-    Psygres._connect(pg_conn)
-
     pool= loc.get('DB_POOL', False)
 
-    if pool:
-        pool_min= loc.get('DB_POOL_MIN', PsygresPool._pool_min)
-        pool_max= loc.get('DB_POOL_MAX', PsygresPool._pool_max)
-        pool_to= loc.get('DB_POOL_MIN', PsygresPool._pool_timeout)
+    db_one= loc.get('DB_PERSISTENT_ONE', False)
+    Psygres._connect_str= _connectStr(loc)
+    if not pool or db_one:
+        Psygres._connect(pg_conn)
 
-        if pool_min> pool_max:
-            Speaker.log_int_raise("Invalid configuration file: %s",
-                    ERR_COL("DB_POOL_MIN> DB_POOL_MAX !!!"))
+    if not pool:
+        return
 
-        PsygresPool._pool_min= pool_min
+    pool_to= loc.get('DB_POOL_TIMEOUT', PsygresPool._pool_timeout)
+    pool_min= loc.get('DB_POOL_MIN', PsygresPool._pool_min)
+    pool_max= loc.get('DB_POOL_MAX', PsygresPool._pool_max)
+    pool_calm= loc.get('DB_POOL_PATIENT', False)
+    pool_type= loc.get('DB_POOL_TYPE', PsygresPool._pool_type)
+
+    if pool_type == DB_POOL_LIMITED and pool_min> pool_max:
+        Speaker.log_int_raise("Invalid configuration file: %s",
+                ERR_COL("DB_POOL_MIN > DB_POOL_MAX !!!"))
+
+    PsygresPool._pool_min= pool_min
+    if pool_type in  [DB_POOL_ELASTIC, DB_POOL_GROWING]:
+        PsygresPool._pool_max= -1
+    else:
         PsygresPool._pool_max= pool_max
-        PsygresPool._pool_timeout= pool_to
 
-        PsygresPool._init_pool()
+    PsygresPool._pool_timeout= pool_to
+    PsygresPool._pool_type= pool_type
+    PsygresPool._pool_calm= pool_calm
+
+    PsygresPool._init_pool()
+
+    return
 
-    del Psygres._db_initialized
 
 __all__ = [
         'Psygres',
