@@ -67,6 +67,8 @@ def init_mother(cfile, pg_conn= None, fnaming= None):
 
     `fnaming` is a function: auto created methods will be named
     with this function. If None, a default function will be used.
+    The function has the following synopsis: foo(str) -> str
+    The input str is the name of the table.
     """
 
     if not hasattr(DbMother, '_mo_initialized'):
@@ -80,14 +82,17 @@ def init_mother(cfile, pg_conn= None, fnaming= None):
 
     d= {}
     import speaker
-    execfile(cfile, speaker.__dict__, d)
+    import dbda
+    names_dict= speaker.__dict__.copy()
+    names_dict.update(dbda.__dict__)
+    execfile(cfile, names_dict, d)
 
     # Handle OIDs or not?
     use_oids= d.get('MOTHER_OIDS', True)
     if not use_oids:
-        from dbda import PsygresFly
+        #from dbda import PsygresFly
         del DbMother._sqlPkeysFromOid
-        PsygresFly.exported_methods.remove('lastrowid')
+        dbda.PsygresFly.exported_methods.remove('lastrowid')
         _sqli= DbMother._sqlInsertNoOid
         _i= DbMother._insertNoOid
     else:
@@ -96,10 +101,13 @@ def init_mother(cfile, pg_conn= None, fnaming= None):
 
     sname= d.get('MOTHER_SESSION_NAME', DbMother._mo_session_name)
 
-    from dbda import init_dbda
+    #from dbda import init_dbda
 
     speaker.init_speaker(d)
-    init_dbda(d, pg_conn)
+    dbda.init_dbda(d, pg_conn)
+
+    del speaker
+    del dbda
 
     # Load DB map
     momap= d['MOTHER_MAP']
@@ -303,6 +311,17 @@ class DbMother(_DbMap):
         # Useful var
         tbl= self.table_name
 
+        # Session() ?
+        if session:
+            session._import(self)
+        elif not Psygres._db_initialized:
+            err= ERR_COL('!!!No Session Available!!!')
+            self.log_int_raise("%s You are using the Db Pool, you "
+                               "have disabled the persistent connection, "
+                               "but no session was used to initialize this "
+                               "Mother class (table= %s)", err, ERR_COL(tbl))
+        self.session= session
+
         # self.fields is a list with TABLE fields.
         # self.pkeys is a list with primary keys for the TABLE.
         # You can set these vars in the child class or let
@@ -317,11 +336,6 @@ class DbMother(_DbMap):
         self.fields= frozenset(self.fields)
         self.pkeys= frozenset(self.pkeys)
 
-        # Session() ?
-        if session:
-            session._import(self)
-        self.session= session
-
         # Storing dict values with some controls depending on the 
         # initialization flag....
         self._initStore(store, flag)
@@ -382,9 +396,9 @@ class DbMother(_DbMap):
         """
 
         s=self._store
-        return set([\
-            k for k in self.pkeys\
-            if not s.has_key(k)\
+        return set([                \
+            k for k in self.pkeys   \
+            if not s.has_key(k)     \
             or s[k] in self._mo_false_values])
 
     def _noPKeyFields(self):
@@ -589,6 +603,43 @@ class DbMother(_DbMap):
 ### Core
 ##
 #
+    # safe transactions: the behaviour of Mother
+    # Sessions is different from the persistent 
+    # connections: with the version 0.5.2, commit()
+    # are allowed on MotherSessions.
+    #
+    # A beginTrans, ignored inside sessions, 
+    # followed by a commit(), is a little
+    # disaster, because if we are inside Sessions,
+    # this causes the commit() of the entire session.
+    #
+    # When we are on the persistent connection, instead,
+    # the beginTrans increment the transaction counter
+    # and the commit() simply recover the counter.
+    #
+    # So, we need the following three wrappers to handle
+    # the two different situations in the same way.
+    #
+    # By using them, the result is always the expected
+    # result: inside sessions, the commit is not made.
+    # With a persistent connection, the counter is 
+    # incremented and decremented so that it's always 
+    # the user that decides the moment of the commit().
+    #
+    # Note that these are not public: these wrappers have
+    # to be used only inside DbMother.
+
+    def _safeBeginTrans(self):
+        if not self.session:
+            self.beginTrans()
+
+    def _safeRollback(self):
+        if not self.session:
+            self.rollback()
+
+    def _safeCommit(self):
+        if not self.session:
+            self.commit()
 
 
     def get_triggers(self, flag, when):
@@ -721,6 +772,15 @@ class DbMother(_DbMap):
         # Remember fields changed.
         self._moved |= new_fields
 
+    def getField(self, field, value= None):
+        """ getField(field [, value= None]) --> value 
+
+        Returns the value of the field `field` -if present- 
+        `value` otherwise (default None).
+        """
+
+        return self._store.get(field, value)
+
     def getFields(self, fields= None, autoload= False):
         """ getFields([fields=None,autoload=False]) --> dict
 
@@ -908,12 +968,12 @@ class DbMother(_DbMap):
         f= self._trigger_actions(self, flag)
 
         if not self._triggers_are_initialized() or \
-                not self.get_flag_triggers(flag):
+           not self.get_flag_triggers(flag):
 
             res= f(*args)
 
         else:
-            self.beginTrans()
+            self._safeBeginTrans()
             try:
                 self.log_insane(
                         "Launching %s triggers...",
@@ -933,10 +993,10 @@ class DbMother(_DbMap):
                 self.trigger(flag, MO_AFTER)
 
             except Exception, s:
-                self.rollback()
+                self._safeRollback()
                 self.log_int_raise(str(s))
 
-            self.commit()
+            self._safeCommit()
 
         self.log_info("Action %s.", OKI_COL("completed"))
         return res
@@ -986,8 +1046,7 @@ class DbMother(_DbMap):
         This is a call to load() with safe_mode= False
         """
 
-        return\
-            self.load(fields,safe_mode=False)
+        return  self.load(fields,safe_mode=False)
 
     def refresh(self, d, flag= MO_NOA):
         """ refresh(d[,flag=MO_NOA]) --> None
@@ -1328,9 +1387,9 @@ class MotherManager:
 
         # If a join filter is specified, we need
         # to build the JOIN STATEMENT before the WHERE
-        qry_filter, joining_table= jbuilder and \
+        qry_filter, joining_table= jbuilder and               \
             self._sqlJoinParent(cbuilder, jbuilder, outer) or \
-                ("", "")
+                ('', '')
 
         # Build SQL filter string
         qry_filter+= " WHERE "
@@ -1438,14 +1497,14 @@ class MotherManager:
         which is a dict.
         """
 
-        self.beginTrans()
+        self._safeBeginTrans()
         try:
             res= self.assignRelation_nt(builder_list, params)
-        except Exception,s:
-            self.rollback()
+        except Exception, s:
+            self._safeRollback()
             raise Exception(s)
 
-        self.commit()
+        self._safeCommit()
         return res
             
     def dropRelation(self, builder_list, params= {}):
@@ -1460,16 +1519,16 @@ class MotherManager:
         for builder, d, flag in builder_list:
             new_obj= builder(d, session= self.session)
             fathers.append(new_obj)
-            if flag==MO_DEL:
+            if flag == MO_DEL:
                 fathers_del.append(new_obj)
 
         # create auto relation init child object
-        t_names=[o.table_name for o in fathers]
+        t_names= [ o.table_name for o in fathers ]
         r_obj = self._getRelationObject(t_names)
         
         # Let child take his values from fathers...
-        for father in [self]+fathers:
-            self._fillChildDict(r_obj,father)
+        for father in [self] + fathers:
+            self._fillChildDict(r_obj, father)
 
         r_obj.setFields(params)
         r_obj.delete()
@@ -1815,7 +1874,7 @@ class MotherBox(Psygres):
 
         self.session= session
         self.builder= builder
-        self.momma= builder()
+        self.momma= builder(session= session)
         self.notriggers= notriggers
     
         if filter:
